<?php
/*
 * This is a PHP library that handles calling Picatcha.
 *
 * Copyright (c) 2011 Picatca -- http://picatcha.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 * The Picatcha server URL
 */
define("PICATCHA_API_SERVER", "api.picatcha.com");


/**
 * in case json functions do not exist
 */
if (!function_exists('json_encode')) {
  require_once('JSON.php');
  function json_encode($value=false) {
    $json = new Services_JSON();
    return $json->encode($value);
  }
}

if (!function_exists('json_decode')) {
  require_once('JSON.php');
  function json_decode($value) {
    $json = new Services_JSON();
    return $json->decode($value);
  }
}


/**
 * Submits an HTTP POST to a Picatcha server
 * @param string $host
 * @param string $path
 * @param array $data
 * @param int port
 *
 * @return array response
 */
function _picatcha_http_post ($host, $path, $data, $port = 80) {
  $http_request  = "POST $path HTTP/1.0\r\n";
  $http_request .= "Host: $host\r\n";
  $http_request .= "User-Agent: Picatcha/PHP\r\n";
  $http_request .= "Content-Length: " . strlen($data) . "\r\n";
  $http_request .= "Content-Type: application/x-www-form-urlencoded;\r\n";
  $http_request .= "\r\n";
  $http_request .= $data;

  $response = '';
  if( false == ( $fs = @fsockopen($host, $port, $errno, $errstr, 10) ) ) {
    die ('Could not open socket');
  }

  fwrite($fs, $http_request);

  while ( !feof($fs) )
    $response .= fgets($fs, 1160); // One TCP-IP packet
  fclose($fs);
  $response = explode("\r\n\r\n", $response, 2);
  return $response;
}


/**
 * Gets the challenge HTML (javascript and non-javascript version).
 *
 * This is called from the browser, and the resulting Picatcha HTML widget
 * is embedded within the HTML form it was called from.
 *
 * @param string $pubkey
 *   A public key for Picatcha
 * @param string $error
 *  The error given by Picatcha (optional, default is null)
 *
 * @return string - The HTML to be embedded in the user's form.
 */
function picatcha_get_html ($pubkey, $error = null){
  $elm_id = 'picatcha';
  $html = '';
  $html .= '<script type="text/javascript" src="http://' . PICATCHA_API_SERVER . '/static/client/jquery.min.js"></script>';
  $html .= '<script type="text/javascript" src="http://' . PICATCHA_API_SERVER . '/static/client/picatcha.js"></script>';
  $html .= '<link href="http://' . PICATCHA_API_SERVER . '/static/client/picatcha.css" rel="stylesheet" type="text/css">';
  $html .= '<script>Picatcha.PUBLIC_KEY="' . $pubkey . '";window.onload=function(){Picatcha.create("' . $elm_id . '", {});};</script>';
  if ( $error != null ) {
    $html .= '<div id="' . $elm_id . '_error">' . $error . '</div>';
  }
  $html .= '<div id="' . $elm_id . '"></div>';
  return $html;
}


/**
 * A PicatchaResponse is returned from picatcha_check_answer()
 */
class PicatchaResponse {
        var $is_valid;
        var $error;
}


/**
  * Calls an HTTP POST function to verify if the user's choices were correct
  *
  * @param string $privkey
  * @param string $remoteip
  * @param string $challenge
  * @param string $response
  * @param array $extra_params
  *   an array of extra variables to post to the server
  *
  * @return ReCaptchaResponse
  */
function picatcha_check_answer ($privkey, $remoteip, $user_agent, $challenge, $response, $extra_params = array()){
  if($privkey == null || $privkey == '') {
    die("To use Picatcha you must get an API key from <a href='http://picatcha.com'>http://picatcha.com</a>");
  }

  if($remoteip == null || $remoteip == '') {
    die("For security reasons, you must pass the remote ip to Picatcha");
  }
  if($user_agent == null || $user_agent == '') {
    die("You must pass the user agent to Picatcha");
  }

  // discard spam submissions
  if ($challenge == null || strlen($challenge) == 0 || $response == null || count($response) == 0) {
    $picatcha_response = new PicatchaResponse();
    $picatcha_response->is_valid = false;
    $picatcha_response->error = 'incorrect-answer';
    return $picatcha_response;
  }

  $params = array (
    'k' => $privkey,
    'ip' => $remoteip,
    'ua' => $user_agent,
    't' => $challenge,
    'r' => $response,
    ) + $extra_params;
  $data = json_encode($params);
  $response = _picatcha_http_post(PICATCHA_API_SERVER, "/v", $data);
  $res = json_decode($response[1], false);

  $picatcha_response = new PicatchaResponse();
  if($res->s){
    $picatcha_response->is_valid = true;
  }
  else {
    $picatcha_response->is_valid = false;
    $picatcha_response->error = $res->e;
  }
  return $picatcha_response;
}


/**
 * Gets a URL where the user can sign up for Picatcha.
 *
 * If your application has a configuration page where you enter a key,
 * you should provide a link using this function.
 *
 * @param string $domain
 *   The domain where the page is hosted
 * @param string
 *   $appname The name of your application
 */
function picatcha_get_signup_url ($domain = null, $appname = null) {
  return "http://picatcha.com/";
}
